---
title: Making your first open source contribution, part 1 - finding projects, issues, and mentored programs
draft: false
tags:
  - open source
date: 2020-02-01
---

I always wanted to contribute to open source, but at the same time I found it to be a bit intimidating & overwhelming. It wasn’t until a few years later that I made my first open source contribution. The paragraph below from [PyCon US](https://us.pycon.org/2019/hatchery/mentoredsprints/) perfectly summarized the struggles I faced when I tried to make my first contribution:

> From a technical perspective, interacting with web-based hosting services (such as GitHub, GitLab, etc.), branching and opening pull requests can be overwhelming if these are not everyday actions of your workflow. The correctness of the code and potential bugs are other common obstacles and fears any newcomer might face. Not to mention the time and frustration accumulated over the task of finding a ‘beginner-friendly’ issue to work on.

I learned a lot of things that I wished I had known about earlier, which I’m going to share through a series of blog posts in three parts:

 1. [[Making your first open source contribution, part 1 - finding projects, issues, and mentored programs]]
 2. [[Making your first open source contribution, part 2 - before you start contributing]]
 3. [[Making your first open source contribution, part 3 - navigating new codebases]]
    
Everyone’s experience is different, though, so I’m mainly sharing about the things I’ve experienced. What works for me might not work for you & vice versa, & it’s totally OK!

If you’re here, you probably already know why you want to contribute to open source. It’s outside of the scope of these posts, so if you want to read more about this, Open Source Guides [has a comprehensive section](https://opensource.guide/how-to-contribute/#why-contribute-to-open-source) on the topic.

---

## Finding projects

### Find projects that you use

I started out by browsing through projects that I use daily such as pandas, matplotlib, & scikit-learn. I find that working on projects that I often use would provide a greater motivation for me to work on it. Using it every day means I’m already pretty familiar with the projects too, which helps.

However, I soon learned that this not that straightforward. These libraries are very popular and the issues that are suitable for beginners seem to be gone within minutes after someone posts the issue. The ones left are the more complicated ones that I wasn’t confident enough I’d be able to tackle.

Now, this depends on you—of course no one’s telling you to NOT work on them! In fact, you’ll probably learn the most from them. But when I started out, I didn’t have much free time since I was still juggling school at the same time. Working on complicated issues would definitely take me a while, so I decided to look for other options.

### Browse websites that help you discover open source projects that need help

I also browsed various websites that help me discover open source projects that need help. A few websites that I’ve tried:

- [CodeTriage](http://codetriage.com/) - [https://www.codetriage.com/](https://www.codetriage.com/)
    
- Mozilla’s Bugs Ahoy - [https://codetribute.mozilla.org/](https://codetribute.mozilla.org/)
    
- Contributors Wanted - [https://dev.to/t/contributorswanted](https://dev.to/t/contributorswanted)
    

These websites exposed me to more open source projects that I had never known before, which is great! While this strategy might work for a lot of people, it didn’t really work for me—since I had never used these projects before, I had to learn everything from zero, & sometimes it was until much later that I realized I wasn’t really interested in working on the project. I guess time was a huge constraint for me, & if I had spent more time I’m sure I would come across something that fits me.

### Follow open source project authors/maintainers on social media

I think I discovered most of the projects I contributed to through Twitter. In fact, I finally made my first open source contribution to [Sarah Drasner’s Object Explorer](https://github.com/sdras/object-explorer/pull/26).

I follow a lot of people who happen to be working on their own open source projects, & since I usually follow people who work on things that intersect my interests, their projects usually interest me too. Plus, they might tweet about their new projects that don’t have a lot of contributors yet, & new projects have a much smaller scope, so the barrier to entry is usually lower.

For example, here is [Andreas Mueller](https://twitter.com/amuellerml) tweeting about his new project:

> If you're interested in becoming involved in my new project dabl [https://t.co/Mvxdh68eZv](https://t.co/Mvxdh68eZv), I tagged some easy first issues. Given that it's pretty early in the development, the barrier to entry should hopefully be much lower than in sklearn for example: [https://t.co/rNors7QjkI](https://t.co/rNors7QjkI)— Andreas Mueller (@amuellerml) [January 28, 2020](https://twitter.com/amuellerml/status/1222202599539167238?ref_src=twsrc%5Etfw)

## Finding issues

Now that you’ve got a project you’d like to work on, it’s time to find an issue that you can work on!

### It’s not always about writing code

One thing I want to emphasize here is that the issue that you pick does not always have something to do with writing code. Writing documentation, translating projects, designing… they all also count as contributing to open source too. Fixing typos is also contributing to open source! There’s room for everyone & every role.

### Browse through beginner-friendly labels

Each issue usually has labels or tags. You can browse through issues that are labeled with labels such as **good-first-issue** or **help-wanted**—these are issues that are approachable for first-time contributors. In fact, GitHub has just recently launched [a feature where it automatically labels beginner-friendly issues](https://github.blog/2020-01-22-how-we-built-good-first-issues/) using the power of machine learning. You can just go to the following link: `github.com/<owner>/<repository>/contribute` . For example, if you want to contribute to electron for the first time, you can go to this link: [https://github.com/electron/electron/contribute](https://github.com/electron/electron/contribute).

If you find that:

- **No one’s working on it**: Awesome! Usually, what you should do next is drop a comment & comment that you’re going to work on this issue. This is a way to let everyone know that someone (that’s you!) is currently working on this issue to avoid double work.
    
- **Someone says they’re working on it, but it’s been months:** If you find an issue where someone has said that they’re working on it but there seems to be no activity for the past few months, feel free to comment and ask if you can take over the issue!
    

One thing to remember is that you can make your own issue, & you can work on it too! Though it might be acceptable to directly make a pull request, keep in mind that some projects, like [rust-wasm](https://github.com/rustwasm/wasm-pack/blob/master/CONTRIBUTING.md), explicitly mention that you have to create your own issue before making a pull request because some discussion might be required. So before you make your own PR, open the issue first to make sure that you’re working on something that people agree should be worked on.

However, before you create that issue…

- **Check out the existing PRs and Issues**There is probably another similar issue open, & you don’t want to create a duplicate issue to save your time & the maintainers’ time.
    
- **Check out the README (more on this on the next post!)** Some open source projects have a guideline on how to create your own issue. This is to make sure that you’ve provided all the information necessary regarding the issue to minimize back-and-forths between you & the maintainers.
    

## Finding mentored programs

Contributing to open source does not have to be a lonely business, especially whenever it’s your first time - it’s totally OK to have help! In fact, having mentors and peers that also learn and work together with you might just accelerate your learning.

How do you find such mentors and peers? There are a few programs that aim to encourage new contributors in open source projects, and in these programs, participants will usually be paired with a mentor that will guide them throughout the program.

### Remote mentorships/internships with mentors

The three programs below are probably the most well-known, though I haven’t personally participated in them because when I learned about them, I was no longer eligible:

- [Google Summer of Code](http://summerofcode.withgoogle.com/): students get the chance to work with an open source organization on a 3-month programming project during their break. Organizations that have participated include Blender Foundation, DBPedia, GNOME, & [others](https://summerofcode.withgoogle.com/archive/).
- [Google Season of Docs](https://developers.google.com/season-of-docs/): same as Google Summer of Code, but focusing on technical writing contributions.
- [Outreachy Internship](https://www.outreachy.org/): internships to work in open source and free software. Open to applicants around the world. Internships can be done remotely. People facing under-representation, systemic bias, or discrimination in the technology industry of their country are encouraged to apply.
    

I’ve personally participated in these programs below, & can totally vouch for them!

- [Increasing Rust’s Reach](https://reach.rust-lang.org/): run by the Rust Programming Language Team to grow Rust’s community of project collaborators and leaders. Participants are individuals who are underrepresented in Rust’s community and the tech industry. Each participant works on a Rust project for three months with a Rust team member. The program does not seem to exist anymore, but you can read about my experience of participating in the program and contributing to Rust [here](https://speakerdeck.com/galuhsahid/rust-reach-and-contributing-to-rust).
- [pandas mentoring](https://github.com/python-sprints/pandas-mentoring): I came across Marc Garcia’s tweet one day:
    

> I think it's time to increase the diversity of the [@pandas_dev](https://twitter.com/pandas_dev?ref_src=twsrc%5Etfw) team.  
>   
> If you need mentoring to become a regular pandas contributor, and you're non-male from outside US/Europe, please DM me.  
>   
> Also DM me if your company wants to provide funds to give scholarships to the mentees.— Marc Garcia (@datapythonista) [July 24, 2019](https://twitter.com/datapythonista/status/1154177854999056384?ref_src=twsrc%5Etfw)

I DM’d him & I was super excited because I use pandas every day! Marc has done a tremendous job on running the mentorship. Since then, I’ve made several contributions to pandas from contributing to its documentation to fixing bugs.

### Mentored sprints

The mentorship programs I mentioned above are mostly done remotely. However, there are also mentored sprints that you can attend if there is one happening in your city.

In a sprint, people gather together to squash bugs or write documentation in a short period a time, such as a day or a weekend. Mentored sprints specifically have mentors that can guide participants on making their contributions. Imagine that by the end of the day, you’ll most likely already make your first contribution while also making new friends in the process!

[PyCon US has mentored sprints last year](https://us.pycon.org/2019/hatchery/mentoredsprints/), and there were a lot of interesting projects that participated from CPython to TensorFlow.

[Python Sprints](https://python-sprints.github.io/) is an non for profit group that gathers coders who want to help improve open source using Python. They have held sprints all around the world, and one of their goals is to help people get started, so if there is one in your city you should definitely drop by! London has held a lot of sprints, so if you want to take a look at how these sprints usually are like [check it out here](https://www.meetup.com/Python-Sprints/).
